;===================================================================
; MIKEFX.ORC: Microphone Orchestra with Global Effects
;===================================================================
; Designed & Coded by:
; Dr. Richard Boulanger
; Professor of Music Synthesis
; Berklee College of Music
; copyright (c)  1995 Analog Devices, Inc.
;===================================================================
; Set Sample/Control Rates & Number of Channels
;===================================================================
sr      =        32000
kr      =        200
ksmps   =        160
nchnls  =        2
;===================================================================
; Initialize Global Sends
;===================================================================
gar     init     0
gae     init     0
;===================================================================
; Generate Waveshapes
;===================================================================
gisin   ftgen   0, 0, 4096, 10, 1
gisweep ftgen   0, 0, 4096, -8, .1, 2048, .9, 2048, .1
;===================================================================
; Microphone Instrument
;===================================================================
        instr    128
irs     =        p4
ies     =        p5
ign     =        p6
ga1,ga2 ins
        outs   ga1 * ign, ga2 * ign
gar,gae maddin ga1, irs, ies
        endin
;===================================================================
; Global Reverb Instrument
;===================================================================
        instr    129
irvbtim =        p4
ign     =        p5
ga1     reverb   gar, irvbtim
        outs12   ga1
gar     =        0
        endin
;===================================================================
; Global Echo Instrument
;===================================================================
        instr    130
ikrvt   =        p4
iloopt  =        p5
ign     =        p6
ga1     comb     gae, ikrvt, iloopt, 0
        outs12   ga1 * ign
gae     =        0
        i.....Prologue21.....Effect0.....ECHO   =       1
print   i.....Prologue21.....Effect0.....ECHO
        endin
;===================================================================
; Microphone Instrument with EQ
;===================================================================
        instr    131
igain   =        p4
ilowcut =        p5
ihicut  =        p6
ilowbst =        p7
ihibst  =        p8
irvbs   =        p9
ga1,ga2 ins
abass   butterlp ga1, ilowcut
atreb   butterhp ga1, ihicut
alowgn  =        abass * ilowbst
ahign   =        atreb * ihibst
amix    =        (ga1 + (alowgn + ahign)) * igain
        outs12   amix
gar     addin    ga1, irvbs
        endin
;===================================================================
; Global Reverb Instrument with Pre-LPF
;===================================================================
        instr    132
icut    =        p5
irvbt   =        p4
ihfroll =        p6
irvbret =        p7
ga1     butterlp gar, icut
ga1     reverb2  ga1, irvbt, ihfroll
ga1     =        ga1 * irvbret
        outs12   ga1
gar     =        0
        i.....Prologue21.....Effect1.....ROOM_REVERB   =       1
print   i.....Prologue21.....Effect1.....ROOM_REVERB
        endin
;===================================================================
        instr    133
icut    =        p5
irvbt   =        p4
ihfroll =        p6
irvbret =        p7
asig    butterlp gar, icut
arev    reverb2  asig, irvbt, ihfroll
        outs12   arev * irvbret
gar     =        0
        endin
;===================================================================
; Global "HALL" Reverb Instrument with Pre - LPF/HPF
;===================================================================
        instr    143
ilocut  =        p5
irvbt   =        p4
irvbret =        p6
ihiroll =        p7
ihicut  =        p8
alow    butterlp gar, ilocut
ahi     butterhp gar, ihicut
asig    =        gar + alow + ahi
arev    reverb2  asig, irvbt, ihiroll
arev    =        arev * irvbret
        outs12   arev
gar     =        0
        i.....Prologue21.....Effect2.....HALL_REVERB   =       1
print   i.....Prologue21.....Effect2.....HALL_REVERB
        endin
;===================================================================
; Global "PLATE" Reverb Instrument with Pre - Low/Hi Cut/Boost
;===================================================================
        instr    144
ilocut  =        p5
irvbt   =        p4
irvbret =        p6
ihiroll =        p7
ihicut  =        p8
ilogain =        p9
ihigain =        p10
asiglo  butterlp gar, ilocut
asighi  butterhp gar, ihicut
alo     =        asiglo * ilogain
ahi     =        asighi * ihigain
amix    =        alo + ahi
arev    reverb2  amix, irvbt, ihiroll
arev    =        arev * irvbret
        outs12   arev
gar     =        0
        i.....Prologue21.....Effect3.....PLATE_REVERB   =       1
print   i.....Prologue21.....Effect3.....PLATE_REVERB
        endin
;===================================================================
; Global MONO CHORUS Instrument with Pre-LPF
;===================================================================
        instr    136
iml     =        p4              ; Chorus Level
icut    =        p5              ; Pre-LPF Cutoff Frequency (in Hz)
ipredel =        p6              ; Chorus Pre-Delay
irat    =        p7              ; Chorus Rate
idep    =        p8              ; Chorus Depth
idel    =        p9              ; Chorus Delay
ifb     =        p10             ; Chorus Feedback
alpf    butterlp gae, icut
asig    chorus1  alpf, irat, irat * 1.02, irat * .97, idel, idel * 1.02, idel * .97, ipredel, idep, ifb, gisin
asigd   delay    asig, .03
        outs     asig * iml, asigd * iml
gae     =        0
        i.....Prologue21.....Effect7.....CHORUS   =       1
print   i.....Prologue21.....Effect7.....CHORUS
        endin
;===================================================================
; Global Ping-Pong Delay Instrument
;===================================================================
        instr    134
iloopl  =        p5 *  2
iloopr  =        p5
ikrvt   =        p4
ign     =        p6
asigl   comb     gae, ikrvt, iloopl, 0
asigr   comb     gae, ikrvt, iloopr, 0
        outs     asigl * ign, (asigr * .5) * ign
gae     =        0
        i.....Prologue21.....Effect8.....DELAY   =       1
print   i.....Prologue21.....Effect8.....DELAY
        endin
;===================================================================
; Global Flange Instrument
;===================================================================
        instr    135
ispeed  =        p4
ims     =        p5
ign     =        p6
idepth  =        ims
ga1     oscili   idepth, ispeed, gisweep
ga2     butterhp gae, 1000
adel1   vdelay   ga2, ga1, ims
adel2   vdelay   adel1, ga1, ims
adel3   vdelay   adel2, ga1, ims
adel4   vdelay   adel3, ga1, ims
adel5   vdelay   adel4, ga1, ims
adel6   vdelay   adel5, ga1, ims
amix    =        adel1 + adel2 + adel3 + adel4 + adel5 + adel6
        outs12   amix * ign
gae     =        0
        i.....Prologue21.....Effect9.....FLANGE   =       1
print   i.....Prologue21.....Effect9.....FLANGE
        endin
;===================================================================
